package org.mercury_im.messenger.android.ui.account;

import android.app.Application;

import androidx.annotation.NonNull;
import androidx.lifecycle.AndroidViewModel;
import androidx.lifecycle.LiveData;
import androidx.lifecycle.MutableLiveData;

import org.mercury_im.messenger.android.MercuryImApplication;
import org.mercury_im.messenger.entity.Account;
import org.mercury_im.messenger.android.ui.MercuryAndroidViewModel;
import org.mercury_im.messenger.core.viewmodel.accounts.AccountsViewModel;
import org.mercury_im.messenger.core.xmpp.state.ConnectionPoolState;

import java.util.logging.Level;
import java.util.logging.Logger;

import javax.inject.Inject;

public class AndroidAccountsViewModel extends AndroidViewModel implements MercuryAndroidViewModel<AccountsViewModel> {

    private static final Logger LOGGER = Logger.getLogger(AndroidAccountsViewModel.class.getName());

    private final MutableLiveData<ConnectionPoolState> connectionPool = new MutableLiveData<>();

    @Inject
    AccountsViewModel viewModel;

    public AndroidAccountsViewModel(@NonNull Application application) {
        super(application);

        MercuryImApplication.getApplication().getAppComponent().inject(this);

        addDisposable(getCommonViewModel().getConnectionPool()
                .subscribe(connectionPool::postValue,
                        error -> LOGGER.log(Level.SEVERE, "Error observing connections", error)));
    }

    public LiveData<ConnectionPoolState> getConnectionPool() {
        return connectionPool;
    }

    public void setAccountEnabled(Account accountModel, boolean enabled) {
        getCommonViewModel().onToggleAccountEnabled(accountModel, enabled);
    }

    @Override
    public AccountsViewModel getCommonViewModel() {
        return viewModel;
    }
}
